// EsteidShellExtension.idl : IDL source for EsteidShellExtension
//

// This file will be processed by the MIDL tool to
// produce the type library (EsteidShellExtension.tlb) and marshalling code.

import "oaidl.idl";
import "ocidl.idl";

[
	object,
	uuid(8BD7CE13-2DB7-4268-8201-CED0626CB94E),
	dual,
	nonextensible,
	helpstring("IEsteidShlExt Interface"),
	pointer_default(unique)
]
interface IEsteidShlExt : IDispatch{
};
[
	uuid(F4748FA8-B59E-43FA-9D53-2380EB141AED),
	version(1.0),
	helpstring("EsteidShellExtension 1.0 Type Library")
]
library EsteidShellExtensionLib
{
	importlib("stdole2.tlb");
	[
		uuid(310AAB39-76FE-401B-8A7F-0F578C5F6AB5),
		helpstring("EsteidShlExt Class")
	]
	coclass EsteidShlExt
	{
		[default] interface IEsteidShlExt;
	};
};
