// Microsoft Visual C++ generated resource script.
//
#include "resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#ifndef APSTUDIO_INVOKED
#include "targetver.h"
#endif
#include "winres.h"
/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// English (U.S.) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
#ifdef _WIN32
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)
#endif //_WIN32

#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE 
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE 
BEGIN
    "#ifndef APSTUDIO_INVOKED\r\n"
    "#include ""targetver.h""\r\n"
    "#endif\r\n"
    "#include ""winres.h""\r\0"
END

3 TEXTINCLUDE 
BEGIN
    "\r\0"
END

#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Version
//

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 0,2,0,0
 PRODUCTVERSION 0,2,0,0
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x4L
 FILETYPE 0x2L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904e4"
        BEGIN
            VALUE "CompanyName", "Smartlink Ltd"
            VALUE "FileDescription", "Estonian ID Card Software explorer integration"
            VALUE "FileVersion", "0.2.0.0"
            VALUE "LegalCopyright", "(c) 2009 Smartlink Ltd.  All rights reserved."
            VALUE "InternalName", "EsteidShellExtension.dll"
            VALUE "OriginalFilename", "EsteidShellExtension.dll"
            VALUE "ProductName", "Estonian ID Card Software"
            VALUE "ProductVersion", "0.2.0.0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1252
    END
END


/////////////////////////////////////////////////////////////////////////////
//
// REGISTRY
//

IDR_ESTEIDEXT           REGISTRY                "EsteidShellExtension.rgs"

/////////////////////////////////////////////////////////////////////////////
//
// String Table
//

STRINGTABLE 
BEGIN
    IDS_PROJNAME            "EsteidShellExtension"
END

#endif    // English (U.S.) resources
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
// Estonian resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ETI)
#ifdef _WIN32
LANGUAGE LANG_ESTONIAN, SUBLANG_DEFAULT
#pragma code_page(1257)
#endif //_WIN32

/////////////////////////////////////////////////////////////////////////////
//
// REGISTRY
//

IDR_ESTEIDSHLEXT        REGISTRY                "EsteidShlExt.rgs"

/////////////////////////////////////////////////////////////////////////////
//
// Bitmap
//

IDB_DIGIDOCBMP          BITMAP                  "resources\\digidoc.bmp"
#endif    // Estonian resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//

/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

